@@ -18,6 +18,7 @@ def make_dispatcher(queue, get_redis, max_ttl = 60, on_exception = None):
     redis         = None
     listener_task = None
     handlers      = {}
+    stopped       = False
 
     processing_map = f"{queue}:processing"
     pending_map    = f"{queue}:pending"
@@ -252,22 +253,24 @@ def make_dispatcher(queue, get_redis, max_ttl = 60, on_exception = None):
         counter = 0
         while True:
             try:
-                task = await dequeue()
+                if not stopped:
+                    task = await dequeue()
 
-                if task is not None:
-                    if task["eta"] <= int(time()):
-                        print(repr_task("handling", task))
+                    if task is not None:
+                        if task["eta"] <= int(time()):
+                            print(repr_task("handling", task))
 
-                        create_task(handle(task))
-                        continue
-                    else:
-                        await re_enqueue(task["id"])
+                            create_task(handle(task))
+                            continue
+                        else:
+                            await re_enqueue(task["id"])
 
                 await sleep(1)
 
-                counter = (counter + 1) % 60
                 if counter == 0:
                     create_task(check_process())
+
+                counter = (counter + 1) % 60
             except Exception as e:
                 await handle_exception(e)
 
@@ -278,11 +281,23 @@ def make_dispatcher(queue, get_redis, max_ttl = 60, on_exception = None):
 
         listener_task = create_task(main())
 
-    def unlisten():
-        nonlocal listener_task
+    async def unlisten():
+        nonlocal listener_task, stopped
 
         assert listener_task is not None, "not listening"
 
+        stopped = True
+
+        r = await get_redis_()
+        while True:
+            total_processing = await r.hlen(processing_map)
+
+            print("processing:", total_processing)
+            if total_processing == 0:
+                break
+
+            await sleep(1)
+
         listener_task.cancel()
         listener_task = None
 
